##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

from Acquisition import Implicit, aq_inner, aq_parent
from ZPublisher import BeforeTraverse

from OFS.SimpleItem import SimpleItem
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo

try:
    from Products.CMFCore.utils import _checkPermission
except:
    _checkPermission = None

from time import time

manage_addWhoOnlineForm=DTMLFile('dtml/addWhoOnline', globals())

def manage_addWhoOnline(self, id='who_online', title='', idle = 300,
                     REQUEST=None):
    """Add a new WhoOnline object with id *id*.
    """

    ob=WhoOnline()
    ob.id=str(id)
    ob.title=title
    ob.idle=int(idle)
    self._setObject(id, ob)
    ob=self._getOb(id)
    ob.manage_afterAdd(ob,self)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)


class WhoOnline(SimpleItem,Implicit):
    """ Who Online
        Tool to store who is online and who is realy active !
    """
    security = ClassSecurityInfo()
    manage_options = (	
                         (
                             { 'label': 'Configure', 'action': 'manage_configure' },
                         ) 
                         + SimpleItem.manage_options
                     )

    meta_type= 'Who Online'


    manage_configure = DTMLFile('dtml/configureWhoOnline', globals(), __name__='manage_configure')

    # add the hook with lower priotity
    def manage_afterAdd(self, item, container):
        if item is self:
            id = self.id
            if callable(id): id = id()
            container = container.this()
            hook = BeforeTraverse.NameCaller(id)
            BeforeTraverse.registerBeforeTraverse(container, hook, self.meta_type, 99)

    # don't forget to remove the hook
    def manage_beforeDelete(self, item, container):
        if item is self:
            BeforeTraverse.unregisterBeforeTraverse(container, self.meta_type)

    def manage_editWhoOnline(self,id=None,title=None,idle=None,REQUEST=None):
        """ configuration """
        if title:
            self.title = title
        if idle:
            self.idle = int(idle)
        if self.REQUEST is not None:
            return self.manage_configure(self, REQUEST, manage_tabs_message="Properties changed.")
	
    def __init__(self):
        self.members_list = {}
        self.anons_list = {}
        self.idle = 300
        self.purge = int(time()) + 600

    # the hook ! store session data and member login if auth
    def __call__(self, client, request, response=None):
        t = int(time()) + self.idle
        session = request.SESSION.getId()
        try:
            #is that really ok ? not sure that is the best way to get active member
            member = request._authUserPW()[0]
        except:
            member = 'Anonymous'
        if member == 'Anonymous':
            self.anons_list[session] = {'time':t}
            for k in self.members_list.keys():
                if self.members_list[k]['session'] == session:
                    del self.members_list[session]
        else:
            self.members_list[member] = {'id':member,'session':session,'time':t}
            if self.anons_list.has_key(session):
                del self.anons_list[session]
        self._purge()

    # Purge anons_list each 600s. Never purge members_list.
    security.declarePrivate( '_purge' )
    def _purge(self):
        t = int(time())
        if self.purge < t:
            self.purge = t + 600
            ks = self.anons_list.keys()
            for k in ks:
                # May be the key was deleted by another hook... ? It's seems bug whithout testing.
                if self.anons_list.has_key(k):
                    if self.anons_list[k]['time'] < t:
                        del self.anons_list[k]

    # for a non CMFSite return a list of users id
    security.declarePrivate( '_getActiveMembers' )
    def _getActiveMembers(self):
        t = int(time())
        online = []
        for m in self.members_list.values():
            if m['time'] > t:
                online.append(m['id'])
        if len(online) > 0:
            online.sort()
            return online
        return None

    # for a CMFSite return a list of member curently active
    security.declarePrivate( '_getActiveCMFMembers' )
    def _getActiveCMFMembers(self):
        t = int(time())
        online = []
        getMemberById = None
        if hasattr(self,'portal_membership'):
            getMemberById = self.portal_membership.getMemberById
        else:
            return None
        if _checkPermission:
            isManager = _checkPermission('Manage portal', self)
        else:
            isManager = None
        members_list = self.members_list.values()
        members_list.sort()
        for m in members_list:
            if m['time'] > t:
                if getMemberById:
                    email = listed = None
                    mb = getMemberById(m['id'])
                    if mb is not None and (isManager or mb.listed):
                        online.append(mb)
        if len(online) > 0:
            return online
        return None

    security.declarePublic( 'getActiveMembers' )
    def getActiveMembers(self):
        """ Return active members """
        if hasattr(self,'portal_membership'): return self._getActiveCMFMembers()
        else: return self._getActiveMembers()

    # return number of active Anonymous
    security.declarePublic( 'onlineVisitors' )
    def getActiveVisitors(self):
        """ Return how many Anonymous visitors are currently online """
        t = int(time())
        online = []
        for m in self.anons_list.values():
            if m['time'] > t:
                online.append(m)
        return len(online)

    security.declarePublic( 'isOnline' )
    def isOnline(self,member):
        """ Return if a member is online """
        t = int(time())
        if self.members_list.has_key(member):
            if self.members_list['member']['time'] > t:
                return 1
        return None

InitializeClass(WhoOnline)
